==> gnome-keyring-3.7.91.tar.xz: gnome-keyring-3.7.91/daemon/dbus/tests/test-secret-search.c <==
/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
/* test-secret-search.c: Test secret search

   Copyright (C) 2013 Red Hat Inc

   The Gnome Keyring Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Gnome Keyring Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the Gnome Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Author: Stef Walter <stefw@gnome.org>
*/

#include "config.h"

#include "gkd-secret-types.h"

#include "egg/egg-testing.h"

#include <gcr/gcr-base.h>

#include <glib.h>
#include <glib/gstdio.h>
#include <gio/gio.h>

#include <fcntl.h>

typedef struct {
	GDBusConnection *connection;
	gchar *service_name;
	GPid service_pid;
	gboolean service_available;
	gchar *service_session;
	guint watch_id;
	guint signal_id;
	gchar *directory;
} Test;

static void
on_test_service_appeared (GDBusConnection *connection,
                          const gchar *name,
                          const gchar *name_owner,
                          gpointer user_data)
{
	Test *test = user_data;
	if (!test->connection)
		test->connection = g_object_ref (connection);
	test->service_available = TRUE;
	egg_test_wait_stop ();
}

static void
on_test_service_vanished (GDBusConnection *connection,
                          const gchar *name,
                          gpointer user_data)
{
	Test *test = user_data;
	if (test->service_available) {
		test->service_available = FALSE;
		egg_test_wait_stop ();
	}
}

static void
on_service_spawned (gpointer user_data)
{
	Test *test = user_data;
	int fd;

	g_setenv ("GNOME_KEYRING_TEST_PATH", test->directory, TRUE);
	g_setenv ("GNOME_KEYRING_TEST_SERVICE", test->service_name, TRUE);

	fd = g_open ("/dev/null", O_WRONLY, 0);
	if (fd != -1)
		dup2 (fd, 1);
}

static void
setup (Test *test,
       gconstpointer unused)
{
	GError *error = NULL;
	GVariant *retval;
	GVariant *output;

	gchar *args[] = {
		TOP_BUILDDIR "/daemon/gnome-keyring-daemon",
		"--foreground",
		"--control-directory",
		"/tmp/keyring-test",
		"--components",
		"secrets",
		NULL,
	};

	test->service_name = g_strdup_printf ("org.gnome.keyring.Test.t%d", getpid ());

	test->watch_id = g_bus_watch_name (G_BUS_TYPE_SESSION, test->service_name,
	                                   G_BUS_NAME_WATCHER_FLAGS_NONE,
	                                   on_test_service_appeared,
	                                   on_test_service_vanished,
	                                   test, NULL);

	test->directory = egg_tests_create_scratch_directory (
		SRCDIR "/files/test.keyring",
		NULL);

	if (!g_spawn_async (NULL, args, NULL,
	                    G_SPAWN_LEAVE_DESCRIPTORS_OPEN | G_SPAWN_DO_NOT_REAP_CHILD,
	                    on_service_spawned, test, &test->service_pid, &error)) {
		g_error ("couldn't start gnome-keyring-daemon for testing: %s", error->message);
		g_assert_not_reached ();
	}

	if (!test->service_available) {
		egg_test_wait ();

		if (!test->service_available) {
			g_warning ("Couldn't start gnome-keyring-daemon test service. ");
			g_assert_not_reached ();
		}
	}

	/* Set by on_test_service_appeared */
	g_assert (test->connection != NULL);

	/* Establish a plain session with the daemon */
	retval = g_dbus_connection_call_sync (test->connection,
	                                      test->service_name,
	                                      SECRET_SERVICE_PATH,
	                                      SECRET_SERVICE_INTERFACE,
	                                      "OpenSession",
	                                      g_variant_new ("(s@v)", "plain",
	                                                     g_variant_new_variant (g_variant_new_string (""))),
	                                      G_VARIANT_TYPE ("(vo)"),
	                                      G_DBUS_CALL_FLAGS_NO_AUTO_START,
	                                      -1, NULL, &error);
	g_assert_no_error (error);

	g_variant_get (retval, "(@vo)", &output, &test->service_session);
	g_variant_unref (output);
	g_variant_unref (retval);
}

static void
teardown (Test *test,
          gconstpointer unused)
{
	g_dbus_connection_signal_unsubscribe (test->connection, test->signal_id);

	if (test->service_pid)
		kill (test->service_pid, SIGTERM);

	if (test->service_available) {
		egg_test_wait ();
		if (test->service_available) {
			g_warning ("Couldn't stop gnome-keyring-daemon test service.");
			g_assert_not_reached ();
		}
	}

	if (test->watch_id)
		g_bus_unwatch_name (test->watch_id);

	g_free (test->service_name);
	g_free (test->service_session);

	if (test->connection)
		g_object_unref (test->connection);

	egg_tests_remove_scratch_directory (test->directory);
	g_free (test->directory);
}

static void
test_service_search_items_unlocked_separate (Test *test,
                                             gconstpointer unused)
{
	GVariantBuilder builder;
	GError *error = NULL;
	GVariant *retval;
	GVariant *attrs;
	GVariant *unlocked;
	GVariant *locked;

	g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ss}"));
	attrs = g_variant_builder_end (&builder);

	retval = g_dbus_connection_call_sync (test->connection, test->service_name,
	                                      "/org/freedesktop/secrets",
	                                      SECRET_SERVICE_INTERFACE,
	                                      "SearchItems",
	                                      g_variant_new ("(@a{ss})", attrs),
	                                      G_VARIANT_TYPE ("(aoao)"),
	                                      G_DBUS_CALL_FLAGS_NO_AUTO_START,
	                                      -1, NULL, &error);
	g_assert_no_error (error);

	g_variant_get (retval, "(@ao@ao)", &unlocked, &locked);
	g_variant_unref (retval);

	g_assert_cmpuint (g_variant_n_children (unlocked), ==, 0);
	g_assert_cmpuint (g_variant_n_children (locked), ==, 1);

	g_variant_unref (unlocked);
	g_variant_unref (locked);
}

static void
test_collection_search_items_combined (Test *test,
                                       gconstpointer unused)
{
	GVariantBuilder builder;
	GError *error = NULL;
	GVariant *retval;
	GVariant *attrs;
	GVariant *items;

	g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ss}"));
	attrs = g_variant_builder_end (&builder);

	retval = g_dbus_connection_call_sync (test->connection, test->service_name,
	                                      "/org/freedesktop/secrets/collection/test",
	                                      SECRET_COLLECTION_INTERFACE,
	                                      "SearchItems",
	                                      g_variant_new ("(@a{ss})", attrs),
	                                      G_VARIANT_TYPE ("(ao)"),
	                                      G_DBUS_CALL_FLAGS_NO_AUTO_START,
	                                      -1, NULL, &error);
	g_assert_no_error (error);

	g_variant_get (retval, "(@ao)", &items);
	g_variant_unref (retval);

	g_assert_cmpuint (g_variant_n_children (items), ==, 1);
	g_variant_unref (items);
}

int
main (int argc, char **argv)
{
#if !GLIB_CHECK_VERSION(2,35,0)
	g_type_init ();
#endif
	g_test_init (&argc, &argv, NULL);

	g_test_add ("/secret-search/service-search-items-unlocked-separate", Test, NULL,
	            setup, test_service_search_items_unlocked_separate, teardown);
	g_test_add ("/secret-search/collection-search-items-combined", Test, NULL,
	            setup, test_collection_search_items_combined, teardown);

	return egg_tests_run_with_loop ();
}

==> gnome-keyring_3.10.1.orig.tar.xz: gnome-keyring-3.10.1/daemon/dbus/tests/test-secret-search.c <==
/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
/* test-secret-search.c: Test secret search

   Copyright (C) 2013 Red Hat Inc

   The Gnome Keyring Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Gnome Keyring Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the Gnome Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Author: Stef Walter <stefw@gnome.org>
*/

#include "config.h"

#include "test-service.h"

#include "gkd-secret-types.h"

#include "egg/egg-testing.h"

#include <gcr/gcr-base.h>

#include <glib.h>
#include <glib/gstdio.h>
#include <gio/gio.h>

#include <fcntl.h>

typedef struct {
	TestService service;
} Test;

static void
setup (Test *test,
       gconstpointer unused)
{
	test_service_setup (&test->service);
}

static void
teardown (Test *test,
          gconstpointer unused)
{
	test_service_teardown (&test->service);
}

static void
test_service_search_items_unlocked_separate (Test *test,
                                             gconstpointer unused)
{
	GVariantBuilder builder;
	GError *error = NULL;
	GVariant *retval;
	GVariant *attrs;
	GVariant *unlocked;
	GVariant *locked;

	g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ss}"));
	attrs = g_variant_builder_end (&builder);

	retval = g_dbus_connection_call_sync (test->service.connection,
	                                      test->service.bus_name,
	                                      "/org/freedesktop/secrets",
	                                      SECRET_SERVICE_INTERFACE,
	                                      "SearchItems",
	                                      g_variant_new ("(@a{ss})", attrs),
	                                      G_VARIANT_TYPE ("(aoao)"),
	                                      G_DBUS_CALL_FLAGS_NO_AUTO_START,
	                                      -1, NULL, &error);
	g_assert_no_error (error);

	g_variant_get (retval, "(@ao@ao)", &unlocked, &locked);
	g_variant_unref (retval);

	g_assert_cmpuint (g_variant_n_children (unlocked), ==, 0);
	g_assert_cmpuint (g_variant_n_children (locked), ==, 1);

	g_variant_unref (unlocked);
	g_variant_unref (locked);
}

static void
test_collection_search_items_combined (Test *test,
                                       gconstpointer unused)
{
	GVariantBuilder builder;
	GError *error = NULL;
	GVariant *retval;
	GVariant *attrs;
	GVariant *items;

	g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ss}"));
	attrs = g_variant_builder_end (&builder);

	retval = g_dbus_connection_call_sync (test->service.connection,
	                                      test->service.bus_name,
	                                      "/org/freedesktop/secrets/collection/test",
	                                      SECRET_COLLECTION_INTERFACE,
	                                      "SearchItems",
	                                      g_variant_new ("(@a{ss})", attrs),
	                                      G_VARIANT_TYPE ("(ao)"),
	                                      G_DBUS_CALL_FLAGS_NO_AUTO_START,
	                                      -1, NULL, &error);
	g_assert_no_error (error);

	g_variant_get (retval, "(@ao)", &items);
	g_variant_unref (retval);

	g_assert_cmpuint (g_variant_n_children (items), ==, 1);
	g_variant_unref (items);
}

int
main (int argc, char **argv)
{
#if !GLIB_CHECK_VERSION(2,35,0)
	g_type_init ();
#endif
	g_test_init (&argc, &argv, NULL);

	g_test_add ("/secret-search/service-search-items-unlocked-separate", Test, NULL,
	            setup, test_service_search_items_unlocked_separate, teardown);
	g_test_add ("/secret-search/collection-search-items-combined", Test, NULL,
	            setup, test_collection_search_items_combined, teardown);

	return egg_tests_run_with_loop ();
}
